﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Media.Imaging;
using System.Windows.Media;
using ImageMagick;

namespace Gifitti.GifImageModule
{
    public class GifModel
    {
        public GifImage gifImage { get; private set; }
        public Image originalGif { get; private set; }
        private int numberOfFrames;
        private Image[] frames;

        //Build a Gif Model instace
        //Re create object when a new gif is loaded
        //Keep when working with same gif
        public GifModel(GifImage gifImage)
        {
            this.gifImage = gifImage;
            //Save the original Gif
            originalGif = this.gifImage.gifImage.Clone() as Image;
            frameConstruction(this.gifImage);
        }

        public GifModel(String path)
        {
            this.gifImage = new GifImage(path);
            //Save the original Gif
            originalGif = this.gifImage.gifImage.Clone() as Image;
            frameConstruction(this.gifImage);
        }

        //Construct Frames for the entire Image
        private void frameConstruction(GifImage gifImage)
        {
            numberOfFrames = gifImage.gifImage.GetFrameCount(FrameDimension.Time);
            frames = new Image[numberOfFrames];

            for (int i = 0; i < numberOfFrames; i++)
            {
                originalGif.SelectActiveFrame(FrameDimension.Time, i);
                frames[i] = originalGif.Clone() as Image;
            }
        }

        public void resetToOriginalGif()
        {
            gifImage.gifImage = originalGif;
        }


        public void resizeGif(int newWidth, int newHeight)
        {
            using (MagickImageCollection collection = new MagickImageCollection())
            {
                for (int i = 0; i < frames.Length; i++)
                {
                    collection.Add(new MagickImage(frames[i] as Bitmap));
                    collection[i].Resize(newWidth, newHeight);
                }
                // Optionally reduce colors
                QuantizeSettings settings = new QuantizeSettings();
                settings.Colors = 256;
                collection.Quantize(settings);

                // Optionally optimize the images (images should have the same size).
                collection.Optimize();

                // Save gif
                collection.Write("tempResize.gif");
            }



        }

        public void returnSubGif(int start, int stop)
        {
            using (MagickImageCollection collection = new MagickImageCollection())
            {
                // Add first image and set the animation delay to 100ms
                for (int i = start; i < stop; i++)
                {
                    collection.Add(new ImageMagick.MagickImage(frames[i] as Bitmap));
                    collection[i - start].AnimationDelay = 1;
                }
                // Optionally reduce colors
                QuantizeSettings settings = new QuantizeSettings();
                settings.Colors = 256;
                collection.Quantize(settings);

                // Optionally optimize the images (images should have the same size).
                collection.Optimize();

                // Save gif
                collection.Write("temp.gif");
            }
        }
    }
}
